@@ -51,6 +51,12 @@ crew_assert <- function(
 #' @param alternative Message about an alternative.
 #' @param condition Either "warning" or "message" to indicate the type
 #'   of condition thrown on deprecation.
+#' @param value Value of the object. Deprecation is skipped
+#'   if `value` is `NULL`.
+#' @param skip_cran Logical of length 1, whether to skip the deprecation
+#'   warning or message on CRAN.
+#' @param frequency Character of length 1, passed to the `.frequency`
+#'   argument of `rlang::warn()`.
 #' @examples
 #' suppressWarnings(
 #'   crew_deprecate(
@@ -65,8 +71,15 @@ crew_deprecate <- function(
   date,
   version,
   alternative,
-  condition = "warning"
+  condition = "warning",
+  value = "x",
+  skip_cran = FALSE,
+  frequency = "always"
 ) {
+  on_cran <- !isTRUE(as.logical(Sys.getenv("NOT_CRAN", "false")))
+  if (is.null(value) || (skip_cran && on_cran)) {
+    return(invisible())
+  }
   message <- sprintf(
     "%s was deprecated on %s (crew version %s). Alternative: %s.",
     name,
@@ -76,15 +89,20 @@ crew_deprecate <- function(
   )
   if_any(
     condition == "warning",
-    crew_warn(
+    rlang::warn(
       message = message,
-      class = c("crew_deprecate", "crew_warning", "crew")
+      class = c("crew_deprecate", "crew_warning", "crew"),
+      .frequency = frequency,
+      .frequency_id = name
     ),
     rlang::inform(
       message = message,
-      class = c("crew_deprecate", "crew_message", "crew")
+      class = c("crew_deprecate", "crew_message", "crew"),
+      .frequency = frequency,
+      .frequency_id = name
     )
   )
+  invisible()
 }
 
 crew_error <- function(message = NULL) {
@@ -115,18 +133,28 @@ crew_stop <- function(message, class) {
   rlang::abort(message = message, class = class, call = emptyenv())
 }
 
-crew_warn <- function(message, class) {
+crew_warn <- function(message, class, frequency = "always", id = NULL) {
   old <- getOption("rlang_backtrace_on_error")
   on.exit(options(rlang_backtrace_on_error = old))
   options(rlang_backtrace_on_error = "none")
-  rlang::warn(message = message, class = class)
+  rlang::warn(
+    message = message,
+    class = class,
+    .frequency = frequency,
+    .frequency_id = id
+  )
 }
 
-crew_message <- function(message) {
+crew_message <- function(message, frequency = "always", id = NULL) {
   old <- getOption("rlang_backtrace_on_error")
   on.exit(options(rlang_backtrace_on_error = old))
   options(rlang_backtrace_on_error = "none")
-  rlang::inform(message = message, class = c("crew_message", "crew"))
+  rlang::inform(
+    message = message,
+    class = c("crew_message", "crew"),
+    .frequency = frequency,
+    .frequency_id = id
+  )
 }
 
 crew_condition_false <- function(condition) {
