@@ -11,15 +11,15 @@ x$start()
 expect_equal(x$launcher$workers$futile, 0L)
 x$launch(n = 1L)
 # Pause until worker idles out.
-expect_equal(x$launcher$workers$futile, 1L)
+expect_equal(x$launcher$workers$futile, 0L)
 x$launch(n = 1L)
 # Pause until worker idles out.
-expect_equal(x$launcher$workers$futile, 2L)
+expect_equal(x$launcher$workers$futile, 1L)
 x$launcher$seconds_idle <- Inf
 x$push(TRUE)
 x$wait()
-expect_equal(x$launcher$workers$futile, 3L)
-x$launcher$terminate(1L)
+expect_equal(x$launcher$workers$futile, 2L)
+x$launcher$terminate_workers(index = 1L)
 # Pause until worker exits.
 x$launcher$seconds_idle <- 1e-9
 x$launch(n = 1L)
