@@ -3,21 +3,21 @@ crew_test("abstract launcher class", {
   expect_crew_error(out$validate())
 })
 
-crew_test("default terminate_launcher() method", {
-  launcher <- crew_class_launcher$new(
-    name = "my_launcher_name",
-    seconds_launch = 1,
-    seconds_idle = 2,
-    seconds_wall = 3,
-    seconds_exit = 4,
-    tasks_max = 7,
-    tasks_timers = 8,
-    reset_globals = TRUE,
-    reset_packages = FALSE,
-    reset_options = FALSE,
-    garbage_collection = FALSE
+crew_test("default launch_launcher() method", {
+  launcher <- crew_class_launcher$new()
+  out <- launcher$launch_worker(
+    call = "a",
+    name = "b",
+    launcher = "c",
+    worker = 57L,
+    instance = "d"
   )
-  expect_null(launcher$terminate_worker())
+  expect_equal(out$abstract, TRUE)
+})
+
+crew_test("default terminate_launcher() method", {
+  launcher <- crew_class_launcher$new()
+  expect_equal(launcher$terminate_worker(handle = crew_null)$abstract, TRUE)
 })
 
 crew_test("launcher settings", {
@@ -26,7 +26,6 @@ crew_test("launcher settings", {
     seconds_launch = 1,
     seconds_idle = 2,
     seconds_wall = 3,
-    seconds_exit = 4,
     tasks_max = 7,
     tasks_timers = 8,
     reset_globals = TRUE,
@@ -44,7 +43,6 @@ crew_test("launcher settings", {
   expect_equal(settings$idletime, 2000)
   expect_equal(settings$walltime, 3000)
   expect_equal(settings$timerstart, 8)
-  expect_equal(settings$exitlinger, 4000)
   expect_equal(settings$cleanup, 15L)
 })
 
@@ -54,7 +52,6 @@ crew_test("launcher alternative cleanup", {
     seconds_launch = 1,
     seconds_idle = 2,
     seconds_wall = 3,
-    seconds_exit = 4,
     tasks_max = 7,
     tasks_timers = 8,
     reset_globals = FALSE,
@@ -73,7 +70,6 @@ crew_test("launcher alternative cleanup 2", {
     seconds_launch = 1,
     seconds_idle = 2,
     seconds_wall = 3,
-    seconds_exit = 4,
     tasks_max = 7,
     tasks_timers = 8,
     reset_globals = TRUE,
@@ -92,7 +88,6 @@ crew_test("launcher alternative cleanup 3", {
     seconds_launch = 1,
     seconds_idle = 2,
     seconds_wall = 3,
-    seconds_exit = 4,
     tasks_max = 7,
     tasks_timers = 8,
     reset_globals = FALSE,
@@ -113,7 +108,6 @@ crew_test("launcher call", {
     seconds_launch = 1,
     seconds_idle = 0.001,
     seconds_wall = 3,
-    seconds_exit = 4,
     tasks_max = 7,
     tasks_timers = 8,
     reset_globals = TRUE,
@@ -150,12 +144,16 @@ crew_test("launcher start()", {
     colnames(workers),
     cols <- c(
       "handle",
+      "termination",
       "socket",
       "start",
       "launches",
       "futile",
       "launched",
+      "terminated",
       "history",
+      "online",
+      "discovered",
       "assigned",
       "complete"
     )
@@ -193,11 +191,13 @@ crew_test("launcher done()", {
   )
   expect_equal(launcher$workers$assigned, rep(0L, nrow(grid)))
   expect_equal(launcher$workers$complete, rep(0L, nrow(grid)))
-  out <- launcher$done(daemons = daemons)
+  launcher$tally(daemons = daemons)
+  out <- which(launcher$done())
   exp <- c(1L, 2L, 3L, 4L, 7L, 8L, 9L, 10L, 11L, 12L)
   expect_equal(out, exp)
   launcher$workers$launched <- rep(FALSE, nrow(grid))
-  expect_equal(launcher$done(daemons = daemons), integer(0L))
+  launcher$tally(daemons = daemons)
+  expect_equal(which(launcher$done()), integer(0L))
 })
 
 crew_test("launcher tally()", {
@@ -205,19 +205,19 @@ crew_test("launcher tally()", {
   launcher <- crew_class_launcher$new(seconds_launch = 9999)
   launcher$start(sockets = rep("x", nrow(grid)))
   launcher$workers$launched <- grid$launched
-  daemons <- cbind(assigned = rep(7L, nrow(grid)), complete = grid$complete)
+  daemons <- cbind(
+    online = c(1L, 1L, 0L, 0L),
+    instance = c(1L, 0L, 1L, 0L),
+    assigned = rep(7L, nrow(grid)),
+    complete = grid$complete
+  )
+  expect_equal(launcher$workers$online, rep(FALSE, nrow(grid)))
+  expect_equal(launcher$workers$discovered, rep(FALSE, nrow(grid)))
   expect_equal(launcher$workers$assigned, rep(0L, nrow(grid)))
   expect_equal(launcher$workers$complete, rep(0L, nrow(grid)))
   launcher$tally(daemons = daemons)
-  expect_equal(launcher$workers$assigned, c(0L, 0L, 7L, 7L))
-  expect_equal(launcher$workers$complete, c(0L, 0L, 3L, 7L))
-  launcher$workers$launched[1L] <- FALSE
-  launcher$workers$launched[4L] <- TRUE
-  launcher$tally(daemons = daemons)
-  expect_equal(launcher$workers$assigned, c(7L, 0L, 7L, 7L))
-  expect_equal(launcher$workers$complete, c(3L, 0L, 3L, 7L))
-  launcher$workers$launched <- !(launcher$workers$launched)
-  launcher$tally(daemons = daemons)
+  expect_equal(launcher$workers$online, c(TRUE, TRUE, FALSE, FALSE))
+  expect_equal(launcher$workers$discovered, c(TRUE, FALSE, TRUE, FALSE))
   expect_equal(launcher$workers$assigned, c(7L, 7L, 7L, 7L))
   expect_equal(launcher$workers$complete, c(3L, 7L, 3L, 7L))
 })
@@ -230,21 +230,6 @@ crew_test("launcher unlaunched()", {
   expect_equal(launcher$unlaunched(n = 2L), c(2L, 3L))
 })
 
-crew_test("launcher backlogged() and resolved()", {
-  grid <- expand.grid(
-    launched = c(TRUE, FALSE),
-    assigned = c(7L, 7L),
-    complete = c(3L, 7L)
-  )
-  launcher <- crew_class_launcher$new(seconds_launch = 9999)
-  launcher$start(sockets = rep("x", nrow(grid)))
-  for (field in colnames(grid)) {
-    launcher$workers[[field]] <- grid[[field]]
-  }
-  expect_equal(launcher$backlogged(), c(2L, 4L))
-  expect_equal(launcher$resolved(), c(6L, 8L))
-})
-
 crew_test("launcher summary", {
   x <- crew_launcher()
   expect_null(x$summary())
@@ -254,118 +239,28 @@ crew_test("launcher summary", {
   expect_equal(nrow(out), 2L)
   expect_equal(
     sort(colnames(out)),
-    sort(c("worker", "launches", "assigned", "complete"))
+    sort(
+      c("worker", "launches", "online", "discovered", "assigned", "complete")
+    )
   )
   expect_equal(out$worker, c(1L, 2L))
   for (field in c("launches", "assigned", "complete")) {
     expect_equal(out[[field]], c(0L, 0L))
   }
+  for (field in c("online", "discovered")) {
+    expect_equal(out[[field]], c(FALSE, FALSE))
+  }
 })
 
-crew_test("custom launcher", {
+crew_test("deprecate seconds_exit", {
+  suppressWarnings(crew_launcher(seconds_exit = 1))
+  expect_true(TRUE)
+})
+
+crew_test("deprecate throttle()", {
   skip_on_cran()
   skip_on_os("windows")
-  skip_if_not_installed("processx")
-  if (isTRUE(as.logical(Sys.getenv("CI", "false")))) {
-    skip_on_os("mac")
-  }
-  custom_launcher_class <- R6::R6Class(
-    classname = "custom_launcher_class",
-    inherit = crew::crew_class_launcher,
-    public = list(
-      launch_worker = function(call, name, launcher, worker, instance) {
-        bin <- if_any(
-          tolower(Sys.info()[["sysname"]]) == "windows",
-          "R.exe",
-          "R"
-        )
-        path <- file.path(R.home("bin"), bin)
-        processx::process$new(command = path, args = c("-e", call))
-      },
-      terminate_worker = function(handle) {
-        handle$kill()
-      }
-    )
-  )
-  crew_controller_custom <- function(
-    name = "custom controller name",
-    workers = 1L,
-    host = "127.0.0.1",
-    port = NULL,
-    tls = crew::crew_tls(mode = "none"),
-    seconds_interval = 0.5,
-    seconds_timeout = 5,
-    seconds_launch = 30,
-    seconds_idle = Inf,
-    seconds_wall = Inf,
-    seconds_exit = 1,
-    tasks_max = Inf,
-    tasks_timers = 0L,
-    reset_globals = TRUE,
-    reset_packages = FALSE,
-    reset_options = FALSE,
-    garbage_collection = FALSE,
-    launch_max = 5L
-  ) {
-    client <- crew::crew_client(
-      name = name,
-      workers = workers,
-      host = host,
-      port = port,
-      tls = tls,
-      seconds_interval = seconds_interval,
-      seconds_timeout = seconds_timeout
-    )
-    launcher <- custom_launcher_class$new(
-      name = name,
-      seconds_interval = seconds_interval,
-      seconds_launch = seconds_launch,
-      seconds_idle = seconds_idle,
-      seconds_wall = seconds_wall,
-      seconds_exit = seconds_exit,
-      tasks_max = tasks_max,
-      tasks_timers = tasks_timers,
-      reset_globals = reset_globals,
-      reset_packages = reset_packages,
-      reset_options = reset_options,
-      garbage_collection = garbage_collection,
-      launch_max = launch_max,
-      tls = tls
-    )
-    controller <- crew::crew_controller(
-      client = client,
-      launcher = launcher
-    )
-    controller$validate()
-    controller
-  }
-  controller <- crew_controller_custom()
-  controller$start()
-  on.exit({
-    controller$terminate()
-    rm(controller)
-    gc()
-    crew_test_sleep()
-  })
-  controller$push(name = "pid", command = ps::ps_pid())
-  controller$wait(seconds_timeout = 10, seconds_interval = 0.5)
-  out <- controller$pop()$result[[1]]
-  handle <- controller$launcher$workers$handle[[1]]
-  exp <- handle$get_pid()
-  expect_equal(out, exp)
-  expect_true(handle$is_alive())
-  controller$launcher$terminate()
-  crew_retry(
-    ~!handle$is_alive(),
-    seconds_interval = 0.1,
-    seconds_timeout = 5
-  )
-  expect_false(handle$is_alive())
-  walk(x = controller$launcher$done(), f = controller$launcher$rotate)
-  controller$launcher$tally()
-  out <- controller$launcher$summary()
-  for (field in colnames(out)) {
-    expect_equal(out[[field]], 1L)
-  }
-  controller$terminate()
+  x <- crew_launcher()
+  suppressWarnings(x$throttle())
+  expect_true(TRUE)
 })
